package com.example.springdemo.entities;

import jakarta.persistence.*;
import lombok.AllArgsConstructor;
import lombok.Getter;
import lombok.NoArgsConstructor;
import lombok.Setter;
import org.springframework.data.annotation.CreatedDate;

import java.math.BigDecimal;
import java.util.Date;
import java.util.List;

//订单类
@Getter
@Setter
@NoArgsConstructor
@AllArgsConstructor
@Entity
@Table(name = "Orders")
public class Orders {
    @Id
    @GeneratedValue(strategy = GenerationType.IDENTITY)
    private Long id;//订单ID

    @OneToMany(cascade = CascadeType.ALL, fetch = FetchType.EAGER)
    @JoinTable(name = "orders_orderDishes",
            joinColumns = {@JoinColumn(name = "ordersId", referencedColumnName = "id")},
            inverseJoinColumns = {@JoinColumn(name = "orderDishesId", referencedColumnName = "id")})
    private List<OrderDishes> orderDishes;//订单中的菜品

    //多个订单对应一个用户，关系是多对一，订单创建后添加用户ID，可以主动改变关系，因此是主控方
    @ManyToOne(cascade = CascadeType.ALL, fetch = FetchType.LAZY)
    @JoinTable(name = "users_orders",
            joinColumns = {@JoinColumn(name = "ordersId", referencedColumnName = "id")},
            inverseJoinColumns = {@JoinColumn(name = "usersId", referencedColumnName = "id")})
    private Users user;//外键，下单用户

    @Column(name = "allPrice")
    private BigDecimal allPrice;//订单总价

    private String message;//下单备注

    @CreatedDate
    @Column(name = "createDate")
    private Date createdDate;//订单创建时间
}
