package com.example.springdemo.serviceImpl;

import com.example.springdemo.dao.DishesRepository;
import com.example.springdemo.dao.MerchantsPasswordRepository;
import com.example.springdemo.dao.MerchantsRepository;
import com.example.springdemo.entities.Dishes;
import com.example.springdemo.entities.Merchants;
import com.example.springdemo.entities.password.MerchantsPassword;
import com.example.springdemo.service.MerchantsService;
import jakarta.annotation.Resource;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;
import java.util.Optional;

@Service
public class MerchantsServiceImpl implements MerchantsService {
    @Resource
    private MerchantsRepository merchantsRepository;
    @Resource
    private MerchantsPasswordRepository merchantsPasswordRepository;

    @Resource
    private DishesRepository dishesRepository;

    @Override
    @Transactional(rollbackFor = Exception.class)
    //添加商户时添加密码
    public Merchants addMerchant(Merchants merchant, MerchantsPassword merchantPassword) {
        Merchants m = merchantsRepository.save(merchant);
        merchantsPasswordRepository.save(merchantPassword);
        return m;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    //删除商户时删除密码
    public void deleteMerchantById(Long merchantId) {
        merchantsRepository.deleteById(merchantId);
        merchantsPasswordRepository.deleteByMerchantsId(merchantId);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    //修改商户信息
    public Merchants updateMerchant(Merchants merchant) {
        return merchantsRepository.save(merchant);
    }

    @Override
    public List<Merchants> findAllMerchants() {
        return merchantsRepository.findAll();
    }

    @Override
    public Optional<Merchants> findById(Long merchantId) {
        return merchantsRepository.findById(merchantId);
    }

    @Override
    public Optional<Merchants> findByName(String name) {
        return merchantsRepository.findByName(name);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    //修改密码
    public int updateMerchantPasswordById(String password, Long merchantId) {
        return merchantsPasswordRepository.updatePassword(password, merchantId);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Dishes addDish(Dishes dish, Long merchantId) {
        return dishesRepository.save(dish);
    }
}
