package com.example.springdemo.serviceImpl;

import com.example.springdemo.dao.RolesPermissionsRepository;
import com.example.springdemo.dao.RolesRepository;
import com.example.springdemo.entities.RBAC.Roles;
import com.example.springdemo.entities.Users;
import com.example.springdemo.service.RolesService;
import jakarta.annotation.Resource;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;
import java.util.Optional;

@Service
public class RolesServiceImpl implements RolesService {

    @Resource
    private RolesRepository rolesRepository;

    @Resource
    private RolesPermissionsRepository rolesPermissionsRepository;

    @Override
    @Transactional(rollbackFor = Exception.class)
    //创建角色
    public Roles createRole(Roles role) {
        return rolesRepository.save(role);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    //更新角色信息
    public Roles updateRole(Roles role) {
        return rolesRepository.save(role);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    //删除角色时，需要删除角色与权限的关联关系
    public void deleteRoleById(Long roleId) {
        if (this.checkRolesPermissionsIfExist(roleId)) {
            //删除角色与权限的关联关系
            rolesPermissionsRepository.deleteByRolesId(roleId);
        }
        //删除角色
        rolesRepository.deleteById(roleId);

    }

    //检查角色与权限的关联关系是否存在
    private boolean checkRolesPermissionsIfExist(Long roleId) {
        return !rolesPermissionsRepository.findByRolesId(roleId).isEmpty();
    }

    @Override
    public List<Roles> findAllRoles() {
        return rolesRepository.findAll();
    }

    @Override
    public Optional<Roles> findByName(String name) {
        return rolesRepository.findByName(name);
    }

    @Override
    public Optional<Roles> findById(Long roleId) {
        return rolesRepository.findById(roleId);
    }

    //TODO: Users角色管理
    @Override
    public void RemoveRoleFromUser(Long userId, Long roleId) {
    }

    @Override
    public Users AssignRoleToUser(Long userId, Long roleId) {
        return null;
    }

    @Override
    public List<Optional<Roles>> getUserRoleById(Long userId) {
        return List.of();
    }
}
