package com.example.springdemo.serviceImpl;

import com.example.springdemo.dao.UsersPasswordRepository;
import com.example.springdemo.dao.UsersRepository;
import com.example.springdemo.entities.RBAC.Permissions;
import com.example.springdemo.entities.RBAC.Roles;
import com.example.springdemo.entities.Users;
import com.example.springdemo.entities.password.UsersPassword;
import com.example.springdemo.service.PermissionsService;
import com.example.springdemo.service.UsersService;
import jakarta.annotation.Resource;
import org.springframework.data.jpa.repository.Modifying;
import org.springframework.security.core.GrantedAuthority;
import org.springframework.security.core.authority.SimpleGrantedAuthority;
import org.springframework.security.core.userdetails.User;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.security.core.userdetails.UsernameNotFoundException;
import org.springframework.stereotype.Component;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Optional;

@Service
@Component
public class UsersServiceImpl implements UsersService {
    @Resource
    private UsersRepository usersRepository;

    @Resource
    private UsersPasswordRepository usersPasswordRepository;

    @Resource
    private PermissionsService permissionsService;

    @Override
    @Transactional(rollbackFor = Exception.class)
    //添加用户时同步添加密码
    public Users addUser(Users user, UsersPassword userPassword) {
        Users u = usersRepository.save(user);
        usersPasswordRepository.save(userPassword);
        return u;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    //删除用户时同步删除密码
    public void deleteUserById(Long userId) {
        usersRepository.deleteById(userId);
        usersPasswordRepository.deleteByUsersId(userId);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    //删除用户时同步删除密码
    public void deleteUserByName(String name) {
        Long userId;
        //如果用户存在
        if (usersRepository.findByName(name).isPresent()) {
            usersRepository.deleteByName(name);
            //获取用户ID
            userId = usersRepository.findByName(name).get().getId();
            //删除密码
            usersPasswordRepository.deleteByUsersId(userId);
        }
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    //更新用户信息
    public Users updateUser(Users user) {
        return usersRepository.save(user);
    }

    @Override
    public List<Users> findAllUsers() {
        return usersRepository.findAll();
    }

    @Override
    public Optional<Users> findById(Long userId) {
        return usersRepository.findById(userId);
    }

    @Override
    public Optional<Users> findByName(String name) {
        return usersRepository.findByName(name);
    }

    @Override
    @Modifying
    @Transactional(rollbackFor = Exception.class)
    //更新用户密码
    public int updateUserPasswordById(String password, Long userId) {
        return usersPasswordRepository.updatePassword(password, userId);
    }

    //根据用户名获取用户信息
    @Override
    public UserDetails loadUserByUsername(String username) throws UsernameNotFoundException {
        Collection<GrantedAuthority> authorities = new ArrayList<>();
        // 从数据库中取出用户信息
        Users user = usersRepository.findByName(username).orElse(null);
        // 判断用户是否存在
        if (user == null) {
            throw new UsernameNotFoundException("用户名不存在");
        }
        // 取出密码
        String password = usersPasswordRepository.findByUsersId(user.getId()).getPassword();

        // 用户角色
        Roles role = user.getRoles();
        authorities.add(new SimpleGrantedAuthority("ROLE_".concat(role.getName())));

        //用户权限
        List<Permissions> permissions = permissionsService.findPermissionsByRoleId(role.getId());
        for (Permissions permission : permissions) {
            SimpleGrantedAuthority simpleGrantedAuthority = new SimpleGrantedAuthority(permission.getName());
            authorities.add(simpleGrantedAuthority);
        }

        // 返回UserDetails实现类
        return new User(user.getName(), "{noop}" + password, authorities);
    }
}
